Alien = Class{}

function Alien:init(x, y, moveSpeed, bulletSpeed, texturePath)
    self.entity = Entity(x, y, 16, 16, 'alien')
    self.moveSpeed = moveSpeed
    self.bulletSpeed = bulletSpeed
    self.timeInterval = 3
    self.timer = os.time() + self.timeInterval
    self.shootingTimeInterval = 1
    self.shootingTimer = os.time()
    self.texture = love.graphics.newImage(texturePath)
    self.isDead = false
    self.animations = {
        ['idle'] = Animation({
            texture = self.texture,
            frames = {
                love.graphics.newQuad(math.random(0, 1) * 16, 0, 16, 16, self.texture:getDimensions())
            },
            loop = true
        })
    }
    self.animation = self.animations['idle']
    self.currentFrame = self.animation:getCurrentFrame()
    self.sounds = {
        ['shoot'] = love.audio.newSource('Laser_Shoot.wav', 'static'),
        ['existing'] = love.audio.newSource('alien.wav', 'static'),
        ['death'] = love.audio.newSource('Explosion.wav', 'static')
    }
end

function Alien:update(dt)
    self:move(dt)
    self.animation:update(dt)
    self.currentFrame = self.animation:getCurrentFrame()
    if math.random(1, 10) % 2 == 0 and os.time() >= self.shootingTimer then
        table.insert(bullets, 1, Bullet(self.entity.x, self.entity.y, 2, 2, math.cos(math.random(50, 100)) * self.bulletSpeed,
        math.sin(math.random(1, 100)) * self.bulletSpeed, 2, 'alienbullet'))
        self.shootingTimer = self.shootingTimer + self.shootingTimeInterval
        self.sounds['shoot']:play()
    end
    self.sounds['existing']:play()
    self:loopPos()
end

function Alien:draw()
    love.graphics.draw(self.texture, self.currentFrame, self.entity.x, self.entity.y, self.angle, 1.25, 1.25, 8, 8)
end

function Alien:move(dt)
    if os.time() >= self.timer then
        self.entity:move(-self.moveSpeed, math.random(0, 1) * -self.moveSpeed, dt)
        if os.time() >= self.timer + self.timeInterval + 2 then
            self.timer = os.time() + self.timeInterval
        end
    end
end

function Alien:loopPos()
    if self.entity.x >= VIRTUAL_WIDTH then
        self.entity:moveTo(0, self.entity.y)
    elseif self.entity.x <= 0 then 
        self.entity:moveTo(VIRTUAL_WIDTH, self.entity.y)
    end
    if self.entity.y >= VIRTUAL_HEIGHT then
        self.entity:moveTo(self.entity.x, 0)
    elseif self.entity.y <= 0 then 
        self.entity:moveTo(self.entity.x, VIRTUAL_HEIGHT)
    end
end
