Player = Class{}

function Player:init(x, y, moveSpeed, rotateSpeed, bulletSpeed, endingVelRate, endingVelCut, texturePath)
    self.entity = Entity(x, y, 16, 16, 'player')
    self.angle = 0
    self.moveSpeed = moveSpeed
    self.rotateSpeed = rotateSpeed
    self.bulletSpeed = bulletSpeed
    self.endingVelocityX = 0
    self.endingVelocityY = 0
    self.endingVelRate = endingVelRate
    self.endingVelCut = endingVelCut
    self.accelerationx = 0
    self.accelerationy = 0
    self.maxAcceleration = 1
    self.texture = love.graphics.newImage(texturePath)
    self.isDead = false
    self.isReviving = false
    self.lives = 3
    self.animations = {
        ['idle'] = Animation({
            texture = self.texture,
            frames = {
                love.graphics.newQuad(0, 0, 16, 16, self.texture:getDimensions())
            },
            loop = true
        }),
        ['moving'] = Animation({
            texture = self.texture,
            frames = {
                love.graphics.newQuad(0, 0, 16, 16, self.texture:getDimensions()),
                love.graphics.newQuad(16, 0, 16, 16, self.texture:getDimensions()),
                love.graphics.newQuad(16, 0, 16, 16, self.texture:getDimensions())
            },
            loop = true
        })
    }
    self.animation = self.animations['idle']
    self.currentFrame = self.animation:getCurrentFrame()
    self.sounds = {
        ['shoot'] = love.audio.newSource('Laser_Shoot.wav', 'static'),
        ['death'] = love.audio.newSource('Explosion.wav', 'static')
    }
end

function Player:update(dt)
    self:move(dt)
    self.animation:update(dt)
    self.currentFrame = self.animation:getCurrentFrame()
    if love.keyboard.wasPressed('space') then
        table.insert(bullets, 1, Bullet(self.entity.x, self.entity.y, 2, 2, math.cos(self.angle) * self.bulletSpeed,
        math.sin(self.angle) * self.bulletSpeed, 2, 'playerbullet'))
        self.sounds['shoot']:play()
    end
    if love.keyboard.isDown('up') then
        self.animation = self.animations['moving']
    else
        self.animation = self.animations['idle']
    end
    self:loopPos()
end

function Player:draw()
    love.graphics.draw(self.texture, self.currentFrame, self.entity.x, self.entity.y, self.angle, 1, 1, 8, 8)
end

function Player:move(dt)
    if love.keyboard.isDown('right') and love.keyboard.isDown('up') == false and 
    love.keyboard.isDown('down') == false then

        self.accelerationx = 0
        self.accelerationy = 0
        self.angle = self.angle + self.rotateSpeed * dt
        if self.accelerationx >= self.maxAcceleration then
            self.accelerationx = self.accelerationx - 2 * dt
            self.accelerationy = self.accelerationy - 2 * dt
        end
        self:leftOverVel(dt)

    elseif love.keyboard.isDown('left') and love.keyboard.isDown('up') == false and 
    love.keyboard.isDown('down') == false then

        
        self.accelerationx = 0
        self.accelerationy = 0
        self.angle = self.angle - self.rotateSpeed * dt
        if self.accelerationx >= self.maxAcceleration then
            self.accelerationx = self.accelerationx - 2 * dt
            self.accelerationy = self.accelerationy - 2 * dt
        end
        self:leftOverVel(dt)

    elseif love.keyboard.isDown('up') and love.keyboard.isDown('left') == false and 
    love.keyboard.isDown('right') == false then
        if self.accelerationx < self.maxAcceleration then
            self.accelerationx = self.accelerationx + 2 * dt
            self.accelerationy = self.accelerationy + 2 * dt
        end

        self.entity:move(math.cos(self.angle) * self.moveSpeed * self.accelerationx, 
        math.sin(self.angle) * self.moveSpeed * self.accelerationy, dt)

        self.endingVelocityX = math.cos(self.angle) * self.moveSpeed * self.accelerationx / self.endingVelCut
        self.endingVelocityY = math.sin(self.angle) * self.moveSpeed * self.accelerationy / self.endingVelCut

    elseif love.keyboard.isDown('down') and love.keyboard.isDown('left') == false and 
    love.keyboard.isDown('right') == false then

        if self.accelerationx < self.maxAcceleration then
            self.accelerationx = self.accelerationx + 2 * dt
            self.accelerationy = self.accelerationy + 2 * dt
        end

        self.entity:move(-math.cos(self.angle) * self.moveSpeed  * self.accelerationx / 2, 
        -math.sin(self.angle) * self.moveSpeed  * self.accelerationy / 2, dt)

        self.endingVelocityX = -math.cos(self.angle) * self.moveSpeed / self.endingVelCut / 2
        self.endingVelocityY = -math.sin(self.angle) * self.moveSpeed / self.endingVelCut / 2

    else
        if self.accelerationx >= self.maxAcceleration then
            self.accelerationx = self.accelerationx - 2 * dt
            self.accelerationy = self.accelerationy - 2 * dt
        end
        self:leftOverVel(dt)
    end
    if (love.keyboard.wasReleased('up') or love.keyboard.wasReleased('down')) then
        self.acceleration = 0
    end
end

function Player:loopPos()
    if self.entity.x >= VIRTUAL_WIDTH then
        self.entity:moveTo(0, self.entity.y)
    elseif self.entity.x <= 0 then 
        self.entity:moveTo(VIRTUAL_WIDTH, self.entity.y)
    end
    if self.entity.y >= VIRTUAL_HEIGHT then
        self.entity:moveTo(self.entity.x, 0)
    elseif self.entity.y <= 0 then 
        self.entity:moveTo(self.entity.x, VIRTUAL_HEIGHT)
    end
end

function Player:leftOverVel(dt)
    self.entity:move(self.endingVelocityX, self.endingVelocityY, dt)
    if self.endingVelocityX < 0 then
        self.endingVelocityX = self.endingVelocityX + self.endingVelRate
        if self.endingVelocityX > 0 then
            self.endingVelocityX = 0
        end
    elseif self.endingVelocityX > 0 then
        self.endingVelocityX = self.endingVelocityX - self.endingVelRate
        if self.endingVelocityX < 0 then
            self.endingVelocityX = 0
        end
    end
    if self.endingVelocityY < 0 then
        self.endingVelocityY = self.endingVelocityY + self.endingVelRate
        if self.endingVelocityY > 0 then
            self.endingVelocityY = 0
        end
    elseif self.endingVelocityY > 0 then
        self.endingVelocityY = self.endingVelocityY - self.endingVelRate
        if self.endingVelocityY < 0 then
            self.endingVelocityY = 0
        end
    end
end