Class = require 'class'
push = require 'push'
require 'Util'
require 'Animation'
require 'Entity'
require 'Bullet'
require 'Xplosion'
require 'Rock'
require 'Alien'
require 'Player'

VIRTUAL_WIDTH = 720
VIRTUAL_HEIGHT = 720
WINDOW_WIDTH = 720
WINDOW_HEIGHT = 720

fileHighScore = io.open("score.txt", "r")
io.input(fileHighScore)
math.randomseed(os.time())
love.graphics.setDefaultFilter('nearest', 'nearest')

function love.load()
    love.graphics.setFont(love.graphics.newFont('font.ttf', 8))
    push:setupScreen(VIRTUAL_WIDTH, VIRTUAL_HEIGHT, WINDOW_WIDTH, WINDOW_HEIGHT, {
        fullscreen = false,
        resizable = false
    })
    love.window.setTitle('Asteroids')
    love.keyboard.keysPressed = {}
    love.keyboard.keysReleased = {}
    isPaused = false
    scoreString = io.read()
    highscore = tonumber(scoreString)
    io.close(fileHighScore)
    if highscore == nil then
        highscore = 0
        fileHighScore = io.open("score.txt", "w")
        io.output(fileHighScore)
        io.write(highscore)
        io.close(fileHighScore)
    end
    gameState = 'start'
    points = 0
    bullets = {}
    rocks = {}
    aliens = {}
    explosions = {}
    lifeIcon = {
        img = love.graphics.newImage('temp.png'),
        quad = love.graphics.newQuad(0, 0, 16, 16, 36, 16)
    }
    startTime = os.time()
    alienTime = 0
    alienSpawningInterval = 25
    timers = {
        ['startGame'] = 2,
        ['revive'] = 2,
        ['revive2'] = 4
    }
    maxRocks = 10
    player = Player(VIRTUAL_WIDTH / 2, VIRTUAL_HEIGHT / 2, 250, 10, 500, 0.5, 1.1, 'temp.png')
end


function love.update(dt)
    if isPaused == false then
        if gameState == 'start' then
            if love.keyboard.wasPressed('return') then
                gameState = 'startGame'
                startTime = os.time() + timers['startGame']
                player.lives = 3
                points = 0
            end
        elseif gameState == 'startGame' then
            if os.time() >= startTime then
                gameState = 'game'
                alienTime = os.time() + alienSpawningInterval
                if alienSpawningInterval > 15 then
                    alienSpawningInterval = alienSpawningInterval - 1
                end
            end
        elseif gameState == 'game' then
            mainGame(dt)
        else
            i = 1
            while i <= #aliens do 
                table.remove(aliens, i)
            end
            if love.keyboard.wasPressed('return') then
                saveHighscore()
                a = 1
                while a <= #rocks do 
                    table.remove(rocks, i)
                end
                spawnRocks()
                gameState = 'start'
            end
        end
        spawnRocks()
        updateRocks(dt)
    end
end


function love.draw()
    push:apply('start')
    if gameState == 'game' then
        love.graphics.clear(0, 0, 0, 255/255)
        if player.isDead == false then
            player:draw()
        end
        a = 1
        while a < #bullets + 1 do
            if #bullets > 0 then
                bullets[a]:draw()
            end
            a = a + 1
        end
        for i = 1, #aliens do 
            if #aliens > 0 then
                if aliens[i].isDead == false then
                    aliens[i]:draw()
                end
            end
        end
        for i = 1, #explosions do
            if #explosions > 0 then
                explosions[i]:draw()
            end
        end
        for i = 1, player.lives do 
            love.graphics.draw(lifeIcon.img, lifeIcon.quad, 20 * i + 10, 60, math.rad(-90), 1, 1, 8, 8)
        end
        love.graphics.print(points, 20, 20, 0, 2, 2)
    elseif gameState == 'start' then
        love.graphics.print("Asteroids", 140, 40, 0, 6, 6)
    elseif gameState == 'startGame' then
        love.graphics.clear(0, 0, 0, 255/255)
        love.graphics.print("Starting", 180, 40, 0, 6, 6)
    elseif gameState == 'death' then
        love.graphics.print("You Died", 180, 40, 0, 6, 6)
        love.graphics.print("Press Enter For Title Screen", 140, 100, 0, 2, 2)
        if points > highscore then
            love.graphics.print("New Highscore: " .. points, 155, 300, 0, 3, 3)
        else
            love.graphics.print("Highscore: " .. highscore, 155, 300, 0, 3, 3)
            love.graphics.print("Score: " .. points, 155, 400, 0, 3, 3)
        end
    end
    if gameState == 'start' or gameState == 'game' or gameState == 'death' then
        for i = 1, #rocks do
            if #rocks > 0 then
                rocks[i]:draw()
            end
        end
    end
    push:apply('end')
end

function mainGame(dt)
    -- Updating Player
    updatePlayer(dt)
    -- Spawning aliens
    if os.time() >= alienTime then
        table.insert(aliens, 1, Alien(VIRTUAL_WIDTH - 16, math.random(16, VIRTUAL_HEIGHT - 16), 300, 500, 'aliens.png'))
        alienTime = os.time() + alienSpawningInterval
    end
    -- Updating aliens
        updateAliens(dt)
    -- Updating bullets and their respective collisions between asteroids
        updateBullets(dt)
    -- Updating explosion objects
        updateExplosions(dt)
    -- Debugging key shortcuts
    if love.keyboard.wasPressed('m') and love.keyboard.wasPressed('.') then
        for i = 1, #rocks do 
            table.remove(rocks, i)
        end
    elseif love.keyboard.wasPressed('l') then
        player.entity:moveTo(math.random(16, VIRTUAL_WIDTH - 16), math.random(16, VIRTUAL_HEIGHT - 16))
        while player.entity:isColliding(rocks, 'rock').condit == true do
            player.entity:moveTo(math.random(16, VIRTUAL_WIDTH - 16), math.random(16, VIRTUAL_HEIGHT - 16))
        end
    end
end

function updatePlayer(dt)
    -- Checking for collision between players and asteroids
    rockToP = player.entity:isColliding(rocks, 'rock')
    if rockToP.condit == true and player.isDead == false and os.time() > startTime + timers['revive'] then
        player.isDead = true
        if rocks[rockToP.it].size ~= 'small' then
            rocks[rockToP.it]:spawnRocks()
            table.remove(rocks, rockToP.it + 2)
        else
            table.remove(rocks, rockToP.it)
        end
        player.sounds['death']:play()
    end
    -- Checking for collision between players and alien bullets
    bulletToP = player.entity:isColliding(bullets, 'alienbullet')
    if bulletToP.condit == true and player.isDead == false and os.time() > startTime + timers['revive'] then
        player.isDead = true
        player.sounds['death']:play()
        table.remove(bullets, bulletToP.it)
    end
    -- Handling player death
    if player.isDead == false then
        player:update(dt)
    else
        if player.isReviving == false then
            table.insert(explosions, 1, Xplosion(player.entity.x, player.entity.y, 1.5, 'xplod.png'))
            startTime = os.time() + timers['revive2']
            player.isReviving = true
        elseif os.time() >= startTime and player.isReviving then
            player.endingVelocityX = 0
            player.endingVelocityY = 0
            player.entity.x = VIRTUAL_WIDTH / 2
            player.entity.y = VIRTUAL_HEIGHT / 2
            player.lives = player.lives - 1
            if player.lives <= -1 then
                if file then
                    saveHighscore()
                end
                gameState = 'death'
            else
                player.isReviving = false
                startTime = os.time() + timers['revive']
            end
            player.isDead = false
        end
    end
end

function updateAliens(dt)
    for i = 1, #aliens do 
        aliens[i]:update(dt)
    end
    s = 1
    -- Checking for collision between aliens and rocks
    while s < #aliens + 1 do
        alienToR = aliens[s].entity:isColliding(rocks, 'rock')
        if alienToR.condit == true then
            table.remove(aliens, s)
            if rocks[alienToR.it].size ~= 'small' then
                table.insert(explosions, 1, Xplosion(rocks[alienToR.it].entity.x, rocks[alienToR.it].entity.y,
                1.5, 'xplod.png'))
                rocks[alienToR.it]:spawnRocks()
                rocks[alienToR.it + 2].sound:play()
                table.remove(rocks, alienToR.it + 2)
            else
                table.insert(explosions, 1, Xplosion(rocks[alienToR.it].entity.x, rocks[alienToR.it].entity.y,
                1, 'xplod.png'))
                rocks[alienToR.it].sound:play()
                table.remove(rocks, alienToR.it)
            end
            s = s - 1
        end
        s = s + 1
    end

    -- Checking for collisions between aliens and player bullets
    l = 1
    while l < #aliens + 1 do
        alienToB = aliens[l].entity:isColliding(bullets, 'playerbullet')
        if alienToB.condit == true then            
            table.insert(explosions, 1, Xplosion(bullets[alienToB.it].entity.x, bullets[alienToB.it].entity.y,
            1.5, 'xplod.png'))
            table.remove(bullets, alienToB.it)
            table.remove(aliens, l)
            points = points + 200
            l = l - 1
        end
        l = l + 1
    end
end

function spawnRocks()
    if #rocks <= 2 then
        max = math.random(maxRocks / 2, maxRocks)
        for i = 1, max do
            table.insert(rocks, 1, Rock(math.random(80, VIRTUAL_WIDTH - 80), math.random(80, VIRTUAL_HEIGHT - 80),
            math.random(40, 50), math.random(40, 50), math.random(-3, 3), 'rock', 'big'))
        end
        startTime = os.time() + timers['revive']
        maxRocks = maxRocks + 2
    end
end

function updateRocks(dt)
    a = 1
    while a < #rocks + 1 do
        if #rocks > 0 then
            rocks[a]:update(dt)
        end
        a = a + 1
    end
end

function updateBullets(dt)
    i = 1
    while i < #bullets + 1 do
        if #bullets > 0 then
            bullets[i]:update(dt)
            if os.time() >= bullets[i].startTime + bullets[i].lifeTime then
                table.remove(bullets, i)
                i = i - 1
            else
                coll = bullets[i].entity:isColliding(rocks, 'rock')
                if coll.condit then
                    rocks[coll.it].sound:play()
                    if rocks[coll.it].size ~= 'small' then
                        table.insert(explosions, 1, Xplosion(rocks[coll.it].entity.x, rocks[coll.it].entity.y,
                        1.5, 'xplod.png'))
                        rocks[coll.it]:spawnRocks()
                        if bullets[i].entity.type == 'playerbullet' then
                            if rocks[coll.it].size == 'big' then
                                points = points + 20
                            else
                                points = points + 50
                            end
                        end
                        table.remove(rocks, coll.it + 2)
                    else
                        table.insert(explosions, 1, Xplosion(rocks[coll.it].entity.x, rocks[coll.it].entity.y,
                        1, 'xplod.png'))
                        table.remove(rocks, coll.it)
                        if bullets[i].entity.type == 'playerbullet' then
                            points = points + 100
                        end
                    end
                    table.remove(bullets, i)
                    i = i - 1
                end
            end
        end
        i = i + 1
    end
end

function updateExplosions(dt)
    j = 1
    while j < #explosions + 1 do
        if #explosions > 0 then
            explosions[j]:update(dt)
            if explosions[j].currentFrame == #explosions[j].animation.frames then
                table.remove(explosions, j)
                j = j - 1
            end
        end
        j = j + 1
    end
end

function love.keypressed(key)
    if key == 'escape' then
        saveHighscore()
        love.event.quit()
    elseif key == 'p' then
        if isPaused == true then
            isPaused = false
        else
            isPaused = true
        end
    end
    love.keyboard.keysPressed[key] = true
end

function saveHighscore()
    if fileHighScore then
        if points > highscore then
            fileHighScore = io.open("score.txt", "w")
            io.output(fileHighScore)
            io.write(points)
            io.close(fileHighScore)
        end
    end
end

function love.keyboard.wasPressed(key)
    if (love.keyboard.keysPressed[key]) then
        love.keyboard.keysPressed[key] = false
        return true
    else
        return false
    end
end

-- global key released function
function love.keyboard.wasReleased(key)
    if (love.keyboard.keysReleased[key]) then
        love.keyboard.keysReleased[key] = false
        return true
    else
        return false
    end
end


-- called whenever a key is released
function love.keyreleased(key)
    love.keyboard.keysReleased[key] = true
end
